@@ -9,6 +9,8 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * RCS: @(#) $Id: fixstrtod.c,v 1.3 2007/04/16 13:36:34 dkf Exp $
  */
 
 #include <stdio.h>
@@ -20,12 +22,12 @@
  * somes systems (e.g. SunOS 4.1.4) stdlib.h doesn't declare strtod.
  */
 
-extern double strtod();
+extern double strtod(char *, char **);
 
 double
-fixstrtod(string, endPtr)
-    char *string;
-    char **endPtr;
+fixstrtod(
+    char *string,
+    char **endPtr)
 {
     double d;
     d = strtod(string, endPtr);
