@@ -9,14 +9,22 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * RCS: @(#) $Id: tclWinPort.h,v 1.50.2.3 2010/05/25 10:37:18 nijtmans Exp $
  */
 
 #ifndef _TCLWINPORT
 #define _TCLWINPORT
 
-#ifndef _TCLINT
-#   include "tclInt.h"
-#endif
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#undef WIN32_LEAN_AND_MEAN
+
+/*
+ * Ask for the winsock function typedefs, also.
+ */
+#define INCL_WINSOCK_API_TYPEDEFS   1
+#include <winsock2.h>
 
 #ifdef CHECK_UNICODE_CALLS
 #   define _UNICODE
@@ -34,17 +42,36 @@
  *---------------------------------------------------------------------------
  */
 
+#ifdef __CYGWIN__
+#   include <unistd.h>
+#   include <wchar.h>
+#else
+#   include <io.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
-
 #include <errno.h>
 #include <fcntl.h>
 #include <float.h>
-#include <io.h>
 #include <malloc.h>
 #include <process.h>
 #include <signal.h>
 #include <string.h>
+#include <limits.h>
+
+#ifdef __CYGWIN__
+#   include <unistd.h>
+#   ifndef _wcsicmp
+#	define _wcsicmp wcscasecmp
+#   endif
+#else
+#   ifndef strncasecmp
+#	define strncasecmp strnicmp
+#   endif
+#   ifndef strcasecmp
+#	define strcasecmp stricmp
+#   endif
+#endif
 
 /*
  * Need to block out these includes for building extensions with MetroWerks
@@ -63,121 +90,129 @@
 
 #include <time.h>
 
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-#undef WIN32_LEAN_AND_MEAN
-
-/* Compatibility to older visual studio / windows platform SDK */
-#if !defined(MAXULONG_PTR)
-typedef DWORD DWORD_PTR;
-typedef DWORD_PTR * PDWORD_PTR;
-#endif
-
-/*
- * Ask for the winsock function typedefs, also.
- */
-#define INCL_WINSOCK_API_TYPEDEFS   1
-#include <winsock2.h>
-
-#ifdef BUILD_tcl
-#   undef TCL_STORAGE_CLASS
-#   define TCL_STORAGE_CLASS DLLEXPORT
-#endif /* BUILD_tcl */
-
 /*
  * Define EINPROGRESS in terms of WSAEINPROGRESS.
  */
 
-#undef	EINPROGRESS
-#define EINPROGRESS	WSAEINPROGRESS
+#ifndef	EINPROGRESS
+#   define EINPROGRESS	WSAEINPROGRESS
+#endif
 
 /*
- * Define ENOTSUP to a value that will never occur.
+ * If ENOTSUP is not defined, define it to a value that will never occur.
  */
 
-#undef ENOTSUP
-#define ENOTSUP	-1030507
-
-/* Those codes, from Visual Studio 2010, conflict with other values */
-#undef ENODATA
-#undef ENOMSG
-#undef ENOSR
-#undef ENOSTR
-#undef EPROTO
+#ifndef ENOTSUP
+#   define ENOTSUP	-1030507
+#endif
 
 /*
  * The following defines redefine the Windows Socket errors as
  * BSD errors so Tcl_PosixError can do the right thing.
  */
 
-#undef EWOULDBLOCK
-#define EWOULDBLOCK	EAGAIN
-#undef EALREADY
-#define EALREADY	149	/* operation already in progress */
-#undef ENOTSOCK
-#define ENOTSOCK	95	/* Socket operation on non-socket */
-#undef EDESTADDRREQ
-#define EDESTADDRREQ	96	/* Destination address required */
-#undef EMSGSIZE
-#define EMSGSIZE	97	/* Message too long */
-#undef EPROTOTYPE
-#define EPROTOTYPE	98	/* Protocol wrong type for socket */
-#undef ENOPROTOOPT
-#define ENOPROTOOPT	99	/* Protocol not available */
-#undef EPROTONOSUPPORT
-#define EPROTONOSUPPORT 120	/* Protocol not supported */
-#undef ESOCKTNOSUPPORT
-#define ESOCKTNOSUPPORT 121	/* Socket type not supported */
-#undef EOPNOTSUPP
-#define EOPNOTSUPP	122	/* Operation not supported on socket */
-#undef EPFNOSUPPORT
-#define EPFNOSUPPORT	123	/* Protocol family not supported */
-#undef EAFNOSUPPORT
-#define EAFNOSUPPORT	124	/* Address family not supported */
-#undef EADDRINUSE
-#define EADDRINUSE	125	/* Address already in use */
-#undef EADDRNOTAVAIL
-#define EADDRNOTAVAIL 126	/* Can't assign requested address */
-#undef ENETDOWN
-#define ENETDOWN	127	/* Network is down */
-#undef ENETUNREACH
-#define ENETUNREACH	128	/* Network is unreachable */
-#undef ENETRESET
-#define ENETRESET	129	/* Network dropped connection on reset */
-#undef ECONNABORTED
-#define ECONNABORTED	130	/* Software caused connection abort */
-#undef ECONNRESET
-#define ECONNRESET	131	/* Connection reset by peer */
-#undef ENOBUFS
-#define ENOBUFS	132	/* No buffer space available */
-#undef EISCONN
-#define EISCONN	133	/* Socket is already connected */
-#undef ENOTCONN
-#define ENOTCONN	134	/* Socket is not connected */
-#undef ESHUTDOWN
-#define ESHUTDOWN	143	/* Can't send after socket shutdown */
-#undef ETOOMANYREFS
-#define ETOOMANYREFS	144	/* Too many references: can't splice */
-#undef ETIMEDOUT
-#define ETIMEDOUT	145	/* Connection timed out */
-#undef ECONNREFUSED
-#define ECONNREFUSED	146	/* Connection refused */
-#undef ELOOP
-#define ELOOP	90	/* Symbolic link loop */
-#undef EHOSTDOWN
-#define EHOSTDOWN	147	/* Host is down */
-#undef EHOSTUNREACH
-#define EHOSTUNREACH	148	/* No route to host */
-#undef ENOTEMPTY
-#define ENOTEMPTY 	93	/* directory not empty */
-#undef EUSERS
-#define EUSERS	94	/* Too many users (for UFS) */
-#undef EDQUOT
-#define EDQUOT	69	/* Disc quota exceeded */
-#undef ESTALE
-#define ESTALE	151	/* Stale NFS file handle */
-#undef EREMOTE
-#define EREMOTE	66	/* The object is remote */
+#ifndef EWOULDBLOCK
+#   define EWOULDBLOCK	EAGAIN
+#endif
+#ifndef EALREADY
+#   define EALREADY	149	/* operation already in progress */
+#endif
+#ifndef ENOTSOCK
+#   define ENOTSOCK	95	/* Socket operation on non-socket */
+#endif
+#ifndef EDESTADDRREQ
+#   define EDESTADDRREQ	96	/* Destination address required */
+#endif
+#ifndef EMSGSIZE
+#   define EMSGSIZE	97	/* Message too long */
+#endif
+#ifndef EPROTOTYPE
+#   define EPROTOTYPE	98	/* Protocol wrong type for socket */
+#endif
+#ifndef ENOPROTOOPT
+#   define ENOPROTOOPT	99	/* Protocol not available */
+#endif
+#ifndef EPROTONOSUPPORT
+#   define EPROTONOSUPPORT 120	/* Protocol not supported */
+#endif
+#ifndef ESOCKTNOSUPPORT
+#   define ESOCKTNOSUPPORT 121	/* Socket type not supported */
+#endif
+#ifndef EOPNOTSUPP
+#   define EOPNOTSUPP	122	/* Operation not supported on socket */
+#endif
+#ifndef EPFNOSUPPORT
+#   define EPFNOSUPPORT	123	/* Protocol family not supported */
+#endif
+#ifndef EAFNOSUPPORT
+#   define EAFNOSUPPORT	124	/* Address family not supported */
+#endif
+#ifndef EADDRINUSE
+#   define EADDRINUSE	125	/* Address already in use */
+#endif
+#ifndef EADDRNOTAVAIL
+#   define EADDRNOTAVAIL 126	/* Can't assign requested address */
+#endif
+#ifndef ENETDOWN
+#   define ENETDOWN	127	/* Network is down */
+#endif
+#ifndef ENETUNREACH
+#   define ENETUNREACH	128	/* Network is unreachable */
+#endif
+#ifndef ENETRESET
+#   define ENETRESET	129	/* Network dropped connection on reset */
+#endif
+#ifndef ECONNABORTED
+#   define ECONNABORTED	130	/* Software caused connection abort */
+#endif
+#ifndef ECONNRESET
+#   define ECONNRESET	131	/* Connection reset by peer */
+#endif
+#ifndef ENOBUFS
+#   define ENOBUFS	132	/* No buffer space available */
+#endif
+#ifndef EISCONN
+#   define EISCONN	133	/* Socket is already connected */
+#endif
+#ifndef ENOTCONN
+#   define ENOTCONN	134	/* Socket is not connected */
+#endif
+#ifndef ESHUTDOWN
+#   define ESHUTDOWN	143	/* Can't send after socket shutdown */
+#endif
+#ifndef ETOOMANYREFS
+#   define ETOOMANYREFS	144	/* Too many references: can't splice */
+#endif
+#ifndef ETIMEDOUT
+#   define ETIMEDOUT	145	/* Connection timed out */
+#endif
+#ifndef ECONNREFUSED
+#   define ECONNREFUSED	146	/* Connection refused */
+#endif
+#ifndef ELOOP
+#   define ELOOP	90	/* Symbolic link loop */
+#endif
+#ifndef EHOSTDOWN
+#   define EHOSTDOWN	147	/* Host is down */
+#endif
+#ifndef EHOSTUNREACH
+#   define EHOSTUNREACH	148	/* No route to host */
+#endif
+#ifndef ENOTEMPTY
+#   define ENOTEMPTY 	93	/* directory not empty */
+#endif
+#ifndef EUSERS
+#   define EUSERS	94	/* Too many users (for UFS) */
+#endif
+#ifndef EDQUOT
+#   define EDQUOT	69	/* Disc quota exceeded */
+#endif
+#ifndef ESTALE
+#   define ESTALE	151	/* Stale NFS file handle */
+#endif
+#ifndef EREMOTE
+#   define EREMOTE	66	/* The object is remote */
+#endif
 
 /*
  * It is very hard to determine how Windows reacts to attempting to
@@ -185,8 +220,25 @@ typedef DWORD_PTR * PDWORD_PTR;
  * region.  So we fake the error code ourselves.
  */
 
-#undef EOVERFLOW
-#define EOVERFLOW	EFBIG	/* The object couldn't fit in the datatype */
+#ifndef EOVERFLOW
+#   ifdef EFBIG
+#      define EOVERFLOW	EFBIG	/* The object couldn't fit in the datatype */
+#   else /* !EFBIG */
+#      define EOVERFLOW	EINVAL	/* Better than nothing! */
+#   endif /* EFBIG */
+#endif /* !EOVERFLOW */
+
+/*
+ * Signals not known to the standard ANSI signal.h.  These are used
+ * by Tcl_WaitPid() and generic/tclPosixStr.c
+ */
+
+#ifndef SIGTRAP
+#   define SIGTRAP  5
+#endif
+#ifndef SIGBUS
+#   define SIGBUS   10
+#endif
 
 /*
  * Supply definitions for macros to query wait status, if not already
@@ -258,7 +310,7 @@ typedef DWORD_PTR * PDWORD_PTR;
  */
 
 #ifndef S_IFLNK
-#define S_IFLNK        0120000  /* Symbolic Link */
+#   define S_IFLNK        0120000  /* Symbolic Link */
 #endif
 
 #ifndef S_ISREG
@@ -310,11 +362,11 @@ typedef DWORD_PTR * PDWORD_PTR;
  */
 
 #ifndef MAXPATH
-#define MAXPATH MAX_PATH
+#   define MAXPATH MAX_PATH
 #endif /* MAXPATH */
 
 #ifndef MAXPATHLEN
-#define MAXPATHLEN MAXPATH
+#   define MAXPATHLEN MAXPATH
 #endif /* MAXPATHLEN */
 
 /*
@@ -335,15 +387,13 @@ typedef DWORD_PTR * PDWORD_PTR;
  */
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
-#    define environ _environ
-#    if defined(_MSC_VER) && (_MSC_VER < 1600)
-#	define hypot _hypot
-#    endif
-#    define exception _exception
-#    undef EDEADLOCK
-#    if defined(__MINGW32__) && !defined(__MSVCRT__)
+#   define environ _environ
+#   define hypot _hypot
+#   define exception _exception
+#   undef EDEADLOCK
+#   if defined(__MINGW32__) && !defined(__MSVCRT__)
 #	define timezone _timezone
-#    endif
+#   endif
 #endif /* _MSC_VER || __MINGW32__ */
 
 /*
@@ -355,6 +405,28 @@ typedef DWORD_PTR * PDWORD_PTR;
 #   define environ  _environ
 #endif /* __BORLANDC__ */
 
+#ifdef __WATCOMC__
+    /*
+     * OpenWatcom uses a wine derived winsock2.h that is missing the
+     * LPFN_* typedefs.
+     */
+#   define HAVE_NO_LPFN_DECLS
+#   if !defined(__CHAR_SIGNED__)
+#	error "You must use the -j switch to ensure char is signed."
+#   endif
+#endif
+
+
+/*
+ * MSVC 8.0 started to mark many standard C library functions depreciated
+ * including the *printf family and others. Tell it to shut up.
+ * (_MSC_VER is 1200 for VC6, 1300 or 1310 for vc7.net, 1400 for 8.0)
+ */
+#if _MSC_VER >= 1400
+#pragma warning(disable:4996)
+#endif
+
+
 /*
  * There is no platform-specific panic routine for Windows in the Tcl internals.
  */
@@ -363,8 +435,8 @@ typedef DWORD_PTR * PDWORD_PTR;
 
 /*
  *---------------------------------------------------------------------------
- * The following macros and declarations represent the interface between 
- * generic and windows-specific parts of Tcl.  Some of the macros may 
+ * The following macros and declarations represent the interface between
+ * generic and windows-specific parts of Tcl.  Some of the macros may
  * override functions declared in tclInt.h.
  *---------------------------------------------------------------------------
  */
@@ -398,17 +470,30 @@ typedef DWORD_PTR * PDWORD_PTR;
 #   define HAVE_PUTENV_THAT_COPIES 1
 #endif
 
+/*
+ * Older version of Mingw are known to lack a MWMO_ALERTABLE define.
+ */
+#if defined(HAVE_NO_MWMO_ALERTABLE)
+#   define MWMO_ALERTABLE 2
+#endif
+
 /*
  * The following defines wrap the system memory allocation routines for
  * use by tclAlloc.c.
  */
 
-#define TclpSysAlloc(size, isBin)	((void*)HeapAlloc(GetProcessHeap(), \
+#ifdef __CYGWIN__
+#   define TclpSysAlloc(size, isBin)	malloc((size))
+#   define TclpSysFree(ptr)		free((ptr))
+#   define TclpSysRealloc(ptr, size)	realloc((ptr), (size))
+#else
+#   define TclpSysAlloc(size, isBin)	((void*)HeapAlloc(GetProcessHeap(), \
 					    (DWORD)0, (DWORD)size))
-#define TclpSysFree(ptr)		(HeapFree(GetProcessHeap(), \
+#   define TclpSysFree(ptr)		(HeapFree(GetProcessHeap(), \
 					    (DWORD)0, (HGLOBAL)ptr))
-#define TclpSysRealloc(ptr, size)	((void*)HeapReAlloc(GetProcessHeap(), \
+#   define TclpSysRealloc(ptr, size)	((void*)HeapReAlloc(GetProcessHeap(), \
 					    (DWORD)0, (LPVOID)ptr, (DWORD)size))
+#endif
 
 /*
  * The following defines map from standard socket names to our internal
@@ -425,75 +510,21 @@ typedef DWORD_PTR * PDWORD_PTR;
 
 
 /*
- * The following macros have trivial definitions, allowing generic code to 
+ * The following macros have trivial definitions, allowing generic code to
  * address platform-specific issues.
  */
 
 #define TclpReleaseFile(file)	ckfree((char *) file)
 
 /*
- * The following macros and declarations wrap the C runtime library 
+ * The following macros and declarations wrap the C runtime library
  * functions.
  */
 
 #define TclpExit		exit
 
-/*
- * Declarations for Windows-only functions.
- */
-
-EXTERN HANDLE	    TclWinSerialReopen _ANSI_ARGS_(( HANDLE handle,
-			CONST TCHAR *name, DWORD access));
-
-EXTERN Tcl_Channel  TclWinOpenSerialChannel _ANSI_ARGS_((HANDLE handle,
-                        char *channelName, int permissions));
-					 
-EXTERN Tcl_Channel  TclWinOpenConsoleChannel _ANSI_ARGS_((HANDLE handle,
-                        char *channelName, int permissions));
-
-EXTERN Tcl_Channel  TclWinOpenFileChannel _ANSI_ARGS_((HANDLE handle,
-                        char *channelName, int permissions, int appendMode));
-
-EXTERN TclFile TclWinMakeFile _ANSI_ARGS_((HANDLE handle));
-
-/*
- * Platform specific mutex definition used by memory allocators.
- * These mutexes are statically allocated and explicitly initialized.
- * Most modules do not use this, but instead use Tcl_Mutex types and
- * Tcl_MutexLock and Tcl_MutexUnlock that are self-initializing.
- */
-
-#ifdef TCL_THREADS
-typedef CRITICAL_SECTION TclpMutex;
-EXTERN void	TclpMutexInit _ANSI_ARGS_((TclpMutex *mPtr));
-EXTERN void	TclpMutexLock _ANSI_ARGS_((TclpMutex *mPtr));
-EXTERN void	TclpMutexUnlock _ANSI_ARGS_((TclpMutex *mPtr));
-#else /* !TCL_THREADS */
-typedef int TclpMutex;
-#define	TclpMutexInit(a)
-#define	TclpMutexLock(a)
-#define	TclpMutexUnlock(a)
-#endif /* TCL_THREADS */
-
-#ifdef TCL_WIDE_INT_TYPE
-EXTERN Tcl_WideInt	strtoll _ANSI_ARGS_((CONST char *string,
-					     char **endPtr, int base));
-EXTERN Tcl_WideUInt	strtoull _ANSI_ARGS_((CONST char *string,
-					      char **endPtr, int base));
-#endif /* TCL_WIDE_INT_TYPE */
-
 #ifndef INVALID_SET_FILE_POINTER
 #define INVALID_SET_FILE_POINTER 0xFFFFFFFF
 #endif /* INVALID_SET_FILE_POINTER */
 
-#ifndef LABEL_SECURITY_INFORMATION
-#   define LABEL_SECURITY_INFORMATION (0x00000010L)
-#endif
-
-#include "tclPlatDecls.h"
-#include "tclIntPlatDecls.h"
-
-#undef TCL_STORAGE_CLASS
-#define TCL_STORAGE_CLASS DLLIMPORT
-
 #endif /* _TCLWINPORT */
